==> org-lsst-ccs-subsystem-shutter-main-4.12-sources.jar: org/lsst/ccs/subsystem/shutter/Commands.java <==
package org.lsst.ccs.subsystem.shutter;

import java.time.Duration;
import java.util.concurrent.TimeoutException;
import java.util.logging.Logger;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.command.annotations.Command;
import static org.lsst.ccs.command.annotations.Command.CommandType.ACTION;
import static org.lsst.ccs.command.annotations.Command.CommandType.QUERY;
import static org.lsst.ccs.command.annotations.Command.ENGINEERING1;
import static org.lsst.ccs.command.annotations.Command.NORMAL;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.subsystem.motorplatform.bus.AxisStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.ChangeAxisEnable;
import org.lsst.ccs.subsystem.motorplatform.bus.ClearAllFaults;
import org.lsst.ccs.subsystem.motorplatform.bus.ClearAxisFaults;
import org.lsst.ccs.subsystem.motorplatform.bus.DisableAllAxes;
import org.lsst.ccs.subsystem.motorplatform.bus.EnableAllAxes;
import org.lsst.ccs.subsystem.motorplatform.bus.HomeAxis;
import org.lsst.ccs.subsystem.motorplatform.bus.MotorplatformType;
import org.lsst.ccs.subsystem.motorplatform.bus.MoveAxisAbsolute;
import org.lsst.ccs.subsystem.motorplatform.bus.MoveAxisRelative;
import org.lsst.ccs.subsystem.motorplatform.bus.SendAxisStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.SendConfiguration;
import org.lsst.ccs.subsystem.motorplatform.bus.SendControllerStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.StopAllMotion;
import org.lsst.ccs.subsystem.shutter.common.Axis;
import org.lsst.ccs.subsystem.shutter.plc.Calibrate;
import org.lsst.ccs.subsystem.shutter.plc.ChangeBrakeState;
import org.lsst.ccs.subsystem.shutter.statemachine.Channel;
import org.lsst.ccs.subsystem.shutter.statemachine.EventReply;
import org.lsst.ccs.subsystem.shutter.statemachine.SynchronousChannel;

/**
 * Commands subsystem component, accepts commands.
 *
 * @author tether
 */
public class Commands  extends Subsystem implements HasLifecycle {
    private static final Logger LOG = Logger.getLogger(Commands.class.getName());

    private final static int COMMAND_TIMEOUT = 30; // seconds

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile Subsystem subsys;

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile StateMachine machine;

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile Publisher publish;

    private final Channel<EventReply> replyChan;

    public Commands() {
        super("commands", AgentInfo.AgentType.WORKER);
        replyChan = new SynchronousChannel<>();
    }

    ////////// Implement HasLifecycle //////////

    /**
     * Sets the Agent property marking this as a motorplatform subsystem.
     * Must be done before we connect to the buses.
     */
    @Override
    public void postInit() {
        subsys
            .getAgentService(AgentPropertiesService.class)
            .setAgentProperty(
                MotorplatformType.MOTORPLATFORM_TYPE_AGENT_PROPERTY,
                MotorplatformType.SHUTTER.name());
    }

    ////////// End HasLifecycle //////////

    ////////// Veto of transitions to NORMAL operational mode //////////

    /**
     * Veto a transition to NORMAL mode if the shutter isn't ready for it.
     * @return null if the transition is accepted, otherwise a string detailing
     * why the transition was rejected.
     */
    @Override
    public String vetoTransitionToNormalMode() {
        try {
            machine.gotoProd(replyChan);
            final EventReply reply = replyChan.read();
            if (reply.wasAccepted(null)) {
                return null;  // Accept the transition.
            } else {
                return reply.getMessage();  // Reject it.
            }
        } catch (InterruptedException interruptedException) {
            LOG.warning("Unexpected interruption.");
            Thread.currentThread().interrupt();
            return "Unexpected interruption.";
        } catch (TimeoutException timeoutException) {
            // Shouldn't happen.
            LOG.warning("Unexpected timeout.");
            return "Unexpected timeout.";
        }
    }
    ////////// End of veto code //////////

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="Try again to sync with the shutter.",
    timeout=COMMAND_TIMEOUT)
    public void resync() throws InterruptedException {
        machine.resync(replyChan);
        getStateMachineReply();
    }

    private void getStateMachineReply() throws InterruptedException {
        try {
            final EventReply reply = replyChan.read();
            if (reply.wasAccepted(null)) {
                subsys.sendAck(null);
            }
            else {
                subsys.sendNack(reply.getMessage());
            }
        }
        catch (TimeoutException exc) {
            subsys.sendNack("Timeout inside the subsystem.");
        }
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="Start a calibration.",
        timeout=COMMAND_TIMEOUT)
    public void calibrate() throws InterruptedException {
        machine.calibrate(replyChan, new Calibrate());
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="Toggle the PLC safety check flag.",
        timeout=COMMAND_TIMEOUT)
    public void toggleSafetyCheck() throws InterruptedException {
        machine.toggleSafetyCheck(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION,
        description="Take an exposure for a given number of seconds.",
        timeout=COMMAND_TIMEOUT)
    public void takeExposure(final double exposureTime) throws InterruptedException {
        final long millis = Math.round( exposureTime * 1000.0);
        machine.takeExposure(replyChan, Duration.ofMillis(millis));
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="Open the shutter.",
        timeout=COMMAND_TIMEOUT)
    public void openShutter() throws InterruptedException {
        machine.openShutter(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION,
        description="Close the shutter, aborting any exposure.",
        timeout=COMMAND_TIMEOUT)
    public void closeShutter() throws InterruptedException {
        machine.closeShutter(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="Change the brake state on an axis.",
        timeout=COMMAND_TIMEOUT)
    public void changeBrakeState(final String axisName, final boolean brakeEngaged)
    throws InterruptedException
    {
        final Axis ax = Axis.fromName(axisName);
        if (ax == null) {
            subsys.sendNack("Invalid axis name, use '+X' or '-X'.");
        }
        else {
            machine.changeBrakeState(replyChan, ax,
                    brakeEngaged ? ChangeBrakeState.State.ENGAGED : ChangeBrakeState.State.RELEASED);
            getStateMachineReply();
        }
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="Center the blades (power-off prep).",
        timeout=COMMAND_TIMEOUT)
    public void gotoCenter() throws InterruptedException {
        machine.gotoCenter(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Start an axis homing.",
        timeout=COMMAND_TIMEOUT)
    public void homeAxis(final HomeAxis req) throws InterruptedException {
        machine.homeAxis(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Start a move to absolute position.",
        timeout=COMMAND_TIMEOUT)
    public void moveAxisAbsolute(final MoveAxisAbsolute req) throws InterruptedException {
        machine.moveAxisAbsolute(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Start a position change.",
        timeout=COMMAND_TIMEOUT)
    public void moveAxisRelative(final MoveAxisRelative req) throws InterruptedException {
        machine.moveAxisRelative(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Clear faults on both axes.",
        timeout=COMMAND_TIMEOUT)
    public void clearAllFaults(final ClearAllFaults req) throws InterruptedException {
        machine.clearAllFaults(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Enable/disable axis.",
        timeout=COMMAND_TIMEOUT)
    public void changeAxisEnable(final ChangeAxisEnable req) throws InterruptedException {
        machine.changeAxisEnable(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Clear faults for one axis.",
        timeout=COMMAND_TIMEOUT)
    public void clearAxisFaults(final ClearAxisFaults req) throws InterruptedException {
        machine.clearAxisFaults(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Enable both axes.",
        timeout=COMMAND_TIMEOUT)
    public void enableAllAxes(final EnableAllAxes req) throws InterruptedException {
        machine.enableAllAxes(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Disable both axes.",
        timeout=COMMAND_TIMEOUT)
    public void disableAllAxes(final DisableAllAxes req) throws InterruptedException {
        machine.disableAllAxes(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="(Scripting) Emergency stop motion.",
        timeout=COMMAND_TIMEOUT)
    public void stopAllMotion(final StopAllMotion req) throws InterruptedException {
        machine.reset(replyChan);
        getStateMachineReply();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish controller status ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendControllerStatus(SendControllerStatus cmd) {
        publish.publishControllerStatus();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish controller configuration ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendConfiguration(SendConfiguration cmd) {
        publish.publishConfiguration();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish axis status ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendAxisStatus(SendAxisStatus cmd) {
        publish.publishAxisStatus(cmd.getAxisName());
    }

    ////////// Motorplatform command aliases meant for manual use from consoles. //////////
    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Clear faults on both axes.",
        timeout=COMMAND_TIMEOUT)
    public void clearFaults() throws InterruptedException {
        clearAllFaults(new ClearAllFaults());
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="(Console) Emergeny stop motion.",
        timeout=COMMAND_TIMEOUT)
    public void stopAll() throws InterruptedException {
        stopAllMotion(new StopAllMotion());
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Enable an axis.",
        timeout=COMMAND_TIMEOUT)
    public void enable(String axisName) throws InterruptedException {
        changeAxisEnable(new ChangeAxisEnable(axisName, true));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Disable an axis.",
        timeout=COMMAND_TIMEOUT)
    public void disable(String axisName) throws InterruptedException {
        changeAxisEnable(new ChangeAxisEnable(axisName, false));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Start an axis homing.",
        timeout=COMMAND_TIMEOUT)
    public void home(String axisName) throws InterruptedException {
        homeAxis(new HomeAxis(axisName));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Start a move to an absolute position.",
        timeout=COMMAND_TIMEOUT)
    public void moveTo(String axisName, double newPositionMm, double speedMmSec) throws InterruptedException {
        moveAxisAbsolute(new MoveAxisAbsolute(axisName, newPositionMm, speedMmSec));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Start a position change.",
        timeout=COMMAND_TIMEOUT)
    public void moveBy(String axisName, double positionChangeMm, double seconds) throws InterruptedException {
        moveAxisRelative(new MoveAxisRelative(axisName, positionChangeMm,
            Duration.ofMillis((long)Math.rint(seconds * 1e3))));
    }

    @Command(level=NORMAL, type=QUERY, description="(Console) Show status info for an axis.",
        timeout=COMMAND_TIMEOUT)
    public String status(String axisName) {
        final StringBuilder buff = new StringBuilder(1024);
        buff.append("Axis %s%n");
        buff.append("Enabled %s, Moving %s, At Home %s, At Low %s, At High %s%n");
        buff.append("Position %6.1f%n");
        final AxisStatus axstat = publish.getAxisStatus(axisName);
        if (axstat == null) {
            return "Not available at present.";
        }
        for (String fault: axstat.getFaults()) {
            buff.append(fault);
            buff.append("%n");
        }
        return String.format(buff.toString(),
                axstat.getAxisName(),
                axstat.isEnabled(),
                axstat.isMoving(),
                axstat.isAtHome(),
                axstat.isAtLowLimit(),
                axstat.isAtHighLimit(),
                axstat.getPosition());
    }

}

==> org-lsst-ccs-subsystem-shutter-main-4.13-sources.jar: org/lsst/ccs/subsystem/shutter/Commands.java <==
package org.lsst.ccs.subsystem.shutter;

import java.time.Duration;
import java.util.concurrent.TimeoutException;
import java.util.logging.Logger;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.command.annotations.Command;
import static org.lsst.ccs.command.annotations.Command.CommandType.ACTION;
import static org.lsst.ccs.command.annotations.Command.CommandType.QUERY;
import static org.lsst.ccs.command.annotations.Command.ENGINEERING1;
import static org.lsst.ccs.command.annotations.Command.NORMAL;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.services.AgentStateService;
import org.lsst.ccs.subsystem.motorplatform.bus.AxisStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.ChangeAxisEnable;
import org.lsst.ccs.subsystem.motorplatform.bus.ClearAllFaults;
import org.lsst.ccs.subsystem.motorplatform.bus.ClearAxisFaults;
import org.lsst.ccs.subsystem.motorplatform.bus.DisableAllAxes;
import org.lsst.ccs.subsystem.motorplatform.bus.EnableAllAxes;
import org.lsst.ccs.subsystem.motorplatform.bus.HomeAxis;
import org.lsst.ccs.subsystem.motorplatform.bus.MotorplatformType;
import org.lsst.ccs.subsystem.motorplatform.bus.MoveAxisAbsolute;
import org.lsst.ccs.subsystem.motorplatform.bus.MoveAxisRelative;
import org.lsst.ccs.subsystem.motorplatform.bus.SendAxisStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.SendConfiguration;
import org.lsst.ccs.subsystem.motorplatform.bus.SendControllerStatus;
import org.lsst.ccs.subsystem.motorplatform.bus.StopAllMotion;
import org.lsst.ccs.subsystem.shutter.common.Axis;
import org.lsst.ccs.subsystem.shutter.common.SoftwareState;
import org.lsst.ccs.subsystem.shutter.plc.Calibrate;
import org.lsst.ccs.subsystem.shutter.plc.ChangeBrakeState;
import org.lsst.ccs.subsystem.shutter.statemachine.Channel;
import org.lsst.ccs.subsystem.shutter.statemachine.EventReply;
import org.lsst.ccs.subsystem.shutter.statemachine.SynchronousChannel;

/**
 * Commands subsystem component, accepts commands.
 *
 * @author tether
 */
public class Commands  extends Subsystem implements HasLifecycle {
    private static final Logger LOG = Logger.getLogger(Commands.class.getName());

    private final static int COMMAND_TIMEOUT = 30; // seconds

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile Subsystem subsys;

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile StateMachine machine;

    @LookupField(strategy=LookupField.Strategy.TREE)
    private volatile Publisher publish;

    private final Channel<EventReply> replyChan;

    public Commands() {
        super("commands", AgentInfo.AgentType.WORKER);
        replyChan = new SynchronousChannel<>();
    }

    ////////// Implement HasLifecycle //////////

    /**
     * Sets the Agent property marking this as a motorplatform subsystem.
     * Must be done before we connect to the buses.
     */
    @Override
    public void postInit() {
        subsys
            .getAgentService(AgentPropertiesService.class)
            .setAgentProperty(
                MotorplatformType.MOTORPLATFORM_TYPE_AGENT_PROPERTY,
                MotorplatformType.SHUTTER.name());
    }

    ////////// End HasLifecycle //////////

    ////////// Veto of transitions to NORMAL operational mode //////////

    /**
     * Veto a transition to NORMAL mode if the shutter isn't in the Closed state in Prod.
     * @return null if the transition is accepted, otherwise a string detailing
     * why the transition was rejected.
     */
    @Override
    public String vetoTransitionToNormalMode() {
        final AgentStateService stateServ = subsys.getAgentService(AgentStateService.class);
        final SoftwareState state =
            (SoftwareState)stateServ.getComponentState("statemachine", SoftwareState.class);
        return state == SoftwareState.CLOSED ? null : "Must be in the Closed state in Prod.";
    }
    ////////// End of veto code //////////

    @Command(autoAck=false, level=NORMAL, type=ACTION,
        description="Try again to sync with the shutter controller.",
        timeout=COMMAND_TIMEOUT)
    public void resync() throws InterruptedException {
        machine.resync(replyChan);
        getStateMachineReply();
    }

    private void getStateMachineReply() throws InterruptedException {
        try {
            final EventReply reply = replyChan.read();
            if (reply.wasAccepted(null)) {
                subsys.sendAck(null);
            }
            else {
                subsys.sendNack(reply.getMessage());
            }
        }
        catch (TimeoutException exc) {
            subsys.sendNack("Timeout inside the subsystem.");
        }
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="Start a calibration.",
        timeout=COMMAND_TIMEOUT)
    public void calibrate() throws InterruptedException {
        machine.calibrate(replyChan, new Calibrate());
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="Toggle the PLC safety check flag.",
        timeout=COMMAND_TIMEOUT)
    public void toggleSafetyCheck() throws InterruptedException {
        machine.toggleSafetyCheck(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION,
        description="Take an exposure for a given number of seconds.",
        timeout=COMMAND_TIMEOUT)
    public void takeExposure(final double exposureTime) throws InterruptedException {
        final long millis = Math.round( exposureTime * 1000.0);
        machine.takeExposure(replyChan, Duration.ofMillis(millis));
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="Open the shutter.",
        timeout=COMMAND_TIMEOUT)
    public void openShutter() throws InterruptedException {
        machine.openShutter(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION,
        description="Close the shutter, aborting any exposure.",
        timeout=COMMAND_TIMEOUT)
    public void closeShutter() throws InterruptedException {
        machine.closeShutter(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="Change the brake state on an axis.",
        timeout=COMMAND_TIMEOUT)
    public void changeBrakeState(final String axisName, final boolean brakeEngaged)
    throws InterruptedException
    {
        final Axis ax = Axis.fromName(axisName);
        if (ax == null) {
            subsys.sendNack("Invalid axis name, use '+X' or '-X'.");
        }
        else {
            machine.changeBrakeState(replyChan, ax,
                    brakeEngaged ? ChangeBrakeState.State.ENGAGED : ChangeBrakeState.State.RELEASED);
            getStateMachineReply();
        }
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="Center the blades (power-off prep).",
        timeout=COMMAND_TIMEOUT)
    public void gotoCenter() throws InterruptedException {
        machine.gotoCenter(replyChan);
        getStateMachineReply();
    }
    
    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="Enter the Prod state.",
        timeout=COMMAND_TIMEOUT)
    public void gotoProd() throws InterruptedException {
        machine.gotoProd(replyChan);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Start an axis homing.",
        timeout=COMMAND_TIMEOUT)
    public void homeAxis(final HomeAxis req) throws InterruptedException {
        machine.homeAxis(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Start a move to absolute position.",
        timeout=COMMAND_TIMEOUT)
    public void moveAxisAbsolute(final MoveAxisAbsolute req) throws InterruptedException {
        machine.moveAxisAbsolute(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Start a position change.",
        timeout=COMMAND_TIMEOUT)
    public void moveAxisRelative(final MoveAxisRelative req) throws InterruptedException {
        machine.moveAxisRelative(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Clear faults on both axes.",
        timeout=COMMAND_TIMEOUT)
    public void clearAllFaults(final ClearAllFaults req) throws InterruptedException {
        machine.clearAllFaults(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Enable/disable axis.",
        timeout=COMMAND_TIMEOUT)
    public void changeAxisEnable(final ChangeAxisEnable req) throws InterruptedException {
        machine.changeAxisEnable(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Scripting) Clear faults for one axis.",
        timeout=COMMAND_TIMEOUT)
    public void clearAxisFaults(final ClearAxisFaults req) throws InterruptedException {
        machine.clearAxisFaults(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Enable both axes.",
        timeout=COMMAND_TIMEOUT)
    public void enableAllAxes(final EnableAllAxes req) throws InterruptedException {
        machine.enableAllAxes(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Scripting) Disable both axes.",
        timeout=COMMAND_TIMEOUT)
    public void disableAllAxes(final DisableAllAxes req) throws InterruptedException {
        machine.disableAllAxes(replyChan, req);
        getStateMachineReply();
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="(Scripting) Emergency stop motion.",
        timeout=COMMAND_TIMEOUT)
    public void stopAllMotion(final StopAllMotion req) throws InterruptedException {
        machine.reset(replyChan);
        getStateMachineReply();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish controller status ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendControllerStatus(SendControllerStatus cmd) {
        publish.publishControllerStatus();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish controller configuration ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendConfiguration(SendConfiguration cmd) {
        publish.publishConfiguration();
    }

    @Command(level=NORMAL, type=QUERY, description="(Scripting) Publish axis status ASAP.",
        timeout=COMMAND_TIMEOUT)
    public void sendAxisStatus(SendAxisStatus cmd) {
        publish.publishAxisStatus(cmd.getAxisName());
    }

    ////////// Motorplatform command aliases meant for manual use from consoles. //////////
    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Clear faults on both axes.",
        timeout=COMMAND_TIMEOUT)
    public void clearFaults() throws InterruptedException {
        clearAllFaults(new ClearAllFaults());
    }

    @Command(autoAck=false, level=NORMAL, type=ACTION, description="(Console) Emergeny stop motion.",
        timeout=COMMAND_TIMEOUT)
    public void stopAll() throws InterruptedException {
        stopAllMotion(new StopAllMotion());
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Enable an axis.",
        timeout=COMMAND_TIMEOUT)
    public void enable(String axisName) throws InterruptedException {
        changeAxisEnable(new ChangeAxisEnable(axisName, true));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Disable an axis.",
        timeout=COMMAND_TIMEOUT)
    public void disable(String axisName) throws InterruptedException {
        changeAxisEnable(new ChangeAxisEnable(axisName, false));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION, description="(Console) Start an axis homing.",
        timeout=COMMAND_TIMEOUT)
    public void home(String axisName) throws InterruptedException {
        homeAxis(new HomeAxis(axisName));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Start a move to an absolute position.",
        timeout=COMMAND_TIMEOUT)
    public void moveTo(String axisName, double newPositionMm, double speedMmSec) throws InterruptedException {
        moveAxisAbsolute(new MoveAxisAbsolute(axisName, newPositionMm, speedMmSec));
    }

    @Command(autoAck=false, level=ENGINEERING1, type=ACTION,
        description="(Console) Start a position change.",
        timeout=COMMAND_TIMEOUT)
    public void moveBy(String axisName, double positionChangeMm, double seconds) throws InterruptedException {
        moveAxisRelative(new MoveAxisRelative(axisName, positionChangeMm,
            Duration.ofMillis((long)Math.rint(seconds * 1e3))));
    }

    @Command(level=NORMAL, type=QUERY, description="(Console) Show status info for an axis.",
        timeout=COMMAND_TIMEOUT)
    public String status(String axisName) {
        final StringBuilder buff = new StringBuilder(1024);
        buff.append("Axis %s%n");
        buff.append("Enabled %s, Moving %s, At Home %s, At Low %s, At High %s%n");
        buff.append("Position %6.1f%n");
        final AxisStatus axstat = publish.getAxisStatus(axisName);
        if (axstat == null) {
            return "Not available at present.";
        }
        for (String fault: axstat.getFaults()) {
            buff.append(fault);
            buff.append("%n");
        }
        return String.format(buff.toString(),
                axstat.getAxisName(),
                axstat.isEnabled(),
                axstat.isMoving(),
                axstat.isAtHome(),
                axstat.isAtLowLimit(),
                axstat.isAtHighLimit(),
                axstat.getPosition());
    }

}
